@@ -1,157 +1,168 @@
 package Catalyst::Controller::HTML::FormFu;
-
-use strict;
-use warnings;
-use Moose  qw( extends with );
-extends 'Catalyst::Controller', 'Class::Accessor::Fast';
-with 'Catalyst::Component::InstancePerContext';
+use Moose;
 
 use HTML::FormFu;
-eval "use HTML::FormFu::MultiForm"; # ignore errors
+use HTML::FormFu::MultiForm;
 use Config::Any;
 use Regexp::Assemble;
-use Scalar::Util qw/ weaken /;
+use Scalar::Util qw/ isweak weaken /;
 use Carp qw/ croak /;
+use MRO::Compat;
 
-our $VERSION = '0.03006';
-$VERSION = eval $VERSION;  # see L<perlmodstyle>
+use namespace::autoclean;
 
-__PACKAGE__->mk_accessors(qw( _html_formfu_config ));
+extends 'Catalyst::Controller';
+with 'Catalyst::Component::InstancePerContext';
+
+our $VERSION = '0.06001';
+$VERSION = eval $VERSION;              # see L<perlmodstyle>
+
+has _html_formfu_config => ( is => 'rw' );
 
 sub build_per_context_instance {
     my ( $self, $c ) = @_;
-    
+
     $self->{c} = $c;
-    
+
     return $self;
 }
 
-sub new {
-    my $class = shift;
-    my ($c) = @_;
-    my $self = $class->NEXT::new(@_);
-    
-    $self->_setup( $c );
-    
-    return $self;
-}
+sub BUILD {}
 
-sub _setup {
-    my ( $self, $app ) = @_;
+after BUILD => sub {
+    my ( $self ) = @_;
 
+    my $app = $self->_app;
     my $self_config   = $self->config->{'Controller::HTML::FormFu'} || {};
-    my $parent_config = $app->config->{'Controller::HTML::FormFu'} || {};
+    my $parent_config = $app->config->{'Controller::HTML::FormFu'}  || {};
 
     my %defaults = (
-        form_method   => 'form',
-        form_stash    => 'form',
-        form_attr     => 'Form',
-        config_attr   => 'FormConfig',
-        method_attr   => 'FormMethod',
-        form_action   => "Catalyst::Controller::HTML::FormFu::Action::Form",
-        config_action => "Catalyst::Controller::HTML::FormFu::Action::FormConfig",
-        method_action => "Catalyst::Controller::HTML::FormFu::Action::FormMethod",
-        
-        multiform_method        => 'multiform',
-        multiform_stash         => 'multiform',
-        multiform_attr          => 'MultiForm',
-        multiform_config_attr   => 'MultiFormConfig',
-        multiform_method_attr   => 'MultiFormMethod',
-        multiform_action        => "Catalyst::Controller::HTML::FormFu::Action::MultiForm",
-        multiform_config_action => "Catalyst::Controller::HTML::FormFu::Action::MultiFormConfig",
-        multiform_method_action => "Catalyst::Controller::HTML::FormFu::Action::MultiFormMethod",
-        
+        request_token_enable          => 0,
+        request_token_field_name      => '_token',
+        request_token_session_key     => '__token',
+        request_token_expiration_time => 3600,
+        form_method                   => 'form',
+        form_stash                    => 'form',
+        form_attr                     => 'Form',
+        config_attr                   => 'FormConfig',
+        method_attr                   => 'FormMethod',
+        form_action => "Catalyst::Controller::HTML::FormFu::Action::Form",
+        config_action =>
+            "Catalyst::Controller::HTML::FormFu::Action::FormConfig",
+        method_action =>
+            "Catalyst::Controller::HTML::FormFu::Action::FormMethod",
+
+        multiform_method      => 'multiform',
+        multiform_stash       => 'multiform',
+        multiform_attr        => 'MultiForm',
+        multiform_config_attr => 'MultiFormConfig',
+        multiform_method_attr => 'MultiFormMethod',
+        multiform_action =>
+            "Catalyst::Controller::HTML::FormFu::Action::MultiForm",
+        multiform_config_action =>
+            "Catalyst::Controller::HTML::FormFu::Action::MultiFormConfig",
+        multiform_method_action =>
+            "Catalyst::Controller::HTML::FormFu::Action::MultiFormMethod",
+
         context_stash => 'context',
-        
+
         model_stash => {},
-        
+
         constructor           => {},
         multiform_constructor => {},
-        
-        config_callback  => 1,
+
+        config_callback => 1,
     );
-    
+
     my %args = ( %defaults, %$parent_config, %$self_config );
-    
-    my $local_path = $app->path_to('root','formfu');
-    
-    if ( !exists $args{constructor}{tt_args}
-        || !exists $args{constructor}{tt_args}{INCLUDE_PATH}
-        && -d $local_path )
+
+    my $local_path = $app->path_to( 'root', 'formfu' );
+
+    if (   !exists $args{constructor}{tt_args}
+        || !exists $args{constructor}{tt_args}{INCLUDE_PATH} && -d $local_path )
     {
         $args{constructor}{tt_args}{INCLUDE_PATH} = [$local_path];
     }
-    
+
     $args{constructor}{query_type} ||= 'Catalyst';
-    
+
     # handle config_file_path
-    if ( exists $args{config_file_path } ) {
+    if ( exists $args{config_file_path} ) {
         warn <<'DEPRECATED';
 config_file_path configuration setting is deprecated,
 use $config->{constructor}{config_file_path} instead.
 DEPRECATED
-        
+
         my $path = delete $args{config_file_path};
-        
+
         $args{constructor}{config_file_path} = $path;
     }
-    
+
     if ( !exists $args{constructor}{config_file_path} ) {
         $args{constructor}{config_file_path} = $app->path_to( 'root', 'forms' );
     }
-    
+
     # build regexp of file extensions
     my $regex_builder = Regexp::Assemble->new;
-    
+
     map { $regex_builder->add($_) } Config::Any->extensions;
-    
+
     $args{_file_ext_regex} = $regex_builder->re;
-    
+
     # save config for use by action classes
     $self->_html_formfu_config( \%args );
-    
+
     # add controller methods
     no strict 'refs';
     *{"$args{form_method}"}      = \&_form;
     *{"$args{multiform_method}"} = \&_multiform;
-}
+};
 
 sub _form {
-    my $self = shift;
-    
-    my $form = HTML::FormFu->new({
-        %{ $self->_html_formfu_config->{constructor} },
-        ( @_ ? %{ $_[0] } : () ),
-    });
-    
-    $self->_common_construction( $form );
-    
+    my $self   = shift;
+    my $config = $self->_html_formfu_config;
+    my $form   = HTML::FormFu->new( {
+            %{ $self->_html_formfu_config->{constructor} },
+            ( @_ ? %{ $_[0] } : () ),
+        } );
+
+    $self->_common_construction($form);
+
+    if ( $config->{request_token_enable} ) {
+        $form->plugins( {
+                type            => 'RequestToken',
+                context         => $config->{context_stash},
+                field_name      => $config->{request_token_field_name},
+                session_key     => $config->{request_token_session_key},
+                expiration_time => $config->{request_token_expiration_time} } );
+    }
+
     return $form;
 }
 
 sub _multiform {
     my $self = shift;
-    
-    my $multi = HTML::FormFu::MultiForm->new({
-        %{ $self->_html_formfu_config->{constructor} },
-        %{ $self->_html_formfu_config->{multiform_constructor} },
-        ( @_ ? %{ $_[0] } : () ),
-    });
-    
-    $self->_common_construction( $multi );
-    
+
+    my $multi = HTML::FormFu::MultiForm->new( {
+            %{ $self->_html_formfu_config->{constructor} },
+            %{ $self->_html_formfu_config->{multiform_constructor} },
+            ( @_ ? %{ $_[0] } : () ),
+        } );
+
+    $self->_common_construction($multi);
+
     return $multi;
 }
 
 sub _common_construction {
     my ( $self, $form ) = @_;
-    
+
     croak "form or multi arg required" if !defined $form;
-    
+
     $form->query( $self->{c}->request );
-    
+
     my $config = $self->_html_formfu_config;
-    
+
     if ( exists $config->{config_file_ext} ) {
         warn <<WARNING;
 Configuration setting 'config_file_ext' has been removed.
@@ -159,9 +170,9 @@ We now use Config::Any's load_stems() which automatically finds files with
 known file extensions.
 WARNING
     }
-    
+
     if ( $config->{config_callback} ) {
-            $form->config_callback({
+        $form->config_callback( {
                 plain_value => sub {
                     return if !defined $_;
                     s{__uri_for\((.+?)\)__}
@@ -170,40 +181,40 @@ WARNING
                      { $self->{c}->path_to( split( '\s*,\s*', $1 ) ) }eg;
                     s{__config\((.+?)\)__}
                      { $self->{c}->config->{$1}  }eg;
-                }
-            });
-            
-            weaken( $self->{c} );
+                    }
+            } );
+
+        weaken( $self->{c} )
+            if !isweak( $self->{c} );
     }
-    
+
     if ( $config->{languages_from_context} ) {
         $form->languages( $self->{c}->languages );
     }
-    
+
     if ( $config->{localize_from_context} ) {
         $form->add_localize_object( $self->{c} );
     }
-    
+
     if ( $config->{default_action_use_name} ) {
         my $action = $self->{c}->uri_for( $self->{c}->{action}->name );
-        
-        $self->{c}->log->debug(
-            "FormFu - Setting default action by name: $action"
-        ) if $self->{c}->debug;
-        
-        $form->action( $action );
+
+        $self->{c}
+            ->log->debug( "FormFu - Setting default action by name: $action" )
+            if $self->{c}->debug;
+
+        $form->action($action);
     }
     elsif ( $config->{default_action_use_path} ) {
-        my $action =
-            $self->{c}->{request}->base . $self->{c}->{request}->path;
-        
-        $self->{c}->log->debug(
-            "FormFu - Setting default action by path: $action"
-        ) if $self->{c}->debug;
-        
-        $form->action( $action );
+        my $action = $self->{c}->{request}->base . $self->{c}->{request}->path;
+
+        $self->{c}
+            ->log->debug( "FormFu - Setting default action by path: $action" )
+            if $self->{c}->debug;
+
+        $form->action($action);
     }
-    
+
     my $context_stash = $config->{context_stash};
     $form->stash->{$context_stash} = $self->{c};
     weaken( $form->stash->{$context_stash} );
@@ -223,16 +234,18 @@ sub create_action {
 
     my $config = $self->_html_formfu_config;
 
-    for my $type (qw/ 
+    for my $type (
+        qw/
         form
         config
         method
         multiform
         multiform_config
-        multiform_method /)
+        multiform_method /
+        )
     {
         my $attr = $config->{"${type}_attr"};
-        
+
         if ( exists $args{attributes}{$attr} ) {
             $args{_attr_params} = delete $args{attributes}{$attr};
         }
@@ -242,7 +255,7 @@ sub create_action {
         else {
             next;
         }
-        
+
         push @{ $args{attributes}{ActionClass} }, $config->{"${type}_action"};
         last;
     }
@@ -256,7 +269,7 @@ __END__
 
 =head1 NAME
 
-Catalyst::Controller::HTML::FormFu
+Catalyst::Controller::HTML::FormFu - Catalyst integration for HTML::FormFu
 
 =head1 SYNOPSIS
 
@@ -332,7 +345,7 @@ Catalyst::Controller::HTML::FormFu
     sub quux : Local : FormMethod('load_form') {
         my ( $self, $c ) = @_;
         
-        # using the FormConfig attribute with an argument is equivalent to:
+        # using the FormMethod attribute with an argument is equivalent to:
         #
         # my $form = $self->form;
         #
@@ -361,6 +374,31 @@ Catalyst::Controller::HTML::FormFu
         # Must return a hash-ref suitable to be fed to $form->populate()
     }
 
+You can also use specially-named actions that will only be called under
+certain circumstances.
+
+    sub edit : Chained('group') : PathPart : Args(0) : FormConfig { }
+    
+    sub edit_FORM_VALID {
+        my ( $self, $c ) = @_;
+        
+        my $form  = $c->stash->{form};
+        my $group = $c->stash->{group};
+        
+        $form->model->update( $group );
+        
+        $c->response->redirect( $c->uri_for( '/group', $group->id ) );
+    }
+    
+    sub edit_FORM_NOT_SUBMITTED {
+        my ( $self, $c ) = @_;
+        
+        my $form  = $c->stash->{form};
+        my $group = $c->stash->{group};
+        
+        $form->model->default_values( $group );
+    }
+
 =head1 METHODS
 
 =head2 form
@@ -381,6 +419,104 @@ return value.
 Note that when using this method, the form's L<query|HTML::FormFu/query> 
 method is not populated with the Catalyst request object.
 
+=head1 SPECIAL ACTION NAMES
+
+An example showing how a complicated action method can be broken down into
+smaller sections, making it clearer which code will be run, and when.
+
+    sub edit : Local : FormConfig {
+        my ( $self, $c ) = @_;
+        
+        my $form  = $c->stash->{form};
+        my $group = $c->stash->{group};
+        
+        $c->detach('/unauthorised') unless $c->user->can_edit( $group );
+        
+        if ( $form->submitted_and_valid ) {
+            $form->model->update( $group );
+            
+            $c->response->redirect( $c->uri_for('/group', $group->id ) );
+            return;
+        }
+        elsif ( !$form->submitted ) {
+            $form->model->default_values( $group );
+        }
+        
+        $self->_add_breadcrumbs_nav( $c, $group );
+    }
+
+Instead becomes...
+
+    sub edit : Local : FormConfig {
+        my ( $self, $c ) = @_;
+        
+        $c->detach('/unauthorised') unless $c->user->can_edit(
+            $c->stash->{group}
+        );
+    }
+    
+    sub edit_FORM_VALID {
+        my ( $self, $c ) = @_;
+        
+        my $group = $c->stash->{group};
+        
+        $c->stash->{form}->model->update( $group );
+        
+        $c->response->redirect( $c->uri_for('/group', $group->id ) );
+    }
+    
+    sub edit_FORM_NOT_SUBMITTED {
+        my ( $self, $c ) = @_;
+        
+        $c->stash->{form}->model->default_values(
+            $c->stash->{group}
+        );
+    }
+    
+    sub edit_FORM_RENDER {
+        my ( $self, $c ) = @_;
+        
+        $self->_add_breadcrumbs_nav( $c, $c->stash->{group} );
+    }
+
+For any action method that uses a C<Form>, C<FormConfig> or C<FormMethod>
+attribute, you can add extra methods that use the naming conventions below.
+
+These methods will be called after the original, plainly named action method.
+
+=head2 _FORM_VALID
+
+Run when the form has been submitted and has no errors.
+
+=head2 _FORM_SUBMITTED
+
+Run when the form has been submitted, regardless of whether or not there was
+errors.
+
+=head2 _FORM_COMPLETE
+
+For MultiForms, is run if the MultiForm is completed.
+
+=head2 _FORM_NOT_VALID
+
+Run when the form has been submitted and there were errors.
+
+=head2 _FORM_NOT_SUBMITTED
+
+Run when the form has not been submitted.
+
+=head2 _FORM_NOT_COMPLETE
+
+For MultiForms, is run if the MultiForm is not completed.
+
+=head2 _FORM_RENDER
+
+For normal C<Form> base classes, this subroutine is run after any of the
+other special methods, unless C<< $form->submitted_and_valid >> is true.
+
+For C<MultiForm> base classes, this subroutine is run after any of the other
+special methods, unless C<< $multi->complete >> is true.
+
 =head1 CUSTOMIZATION
 
 You can set your own config settings, using either your controller config 
@@ -565,6 +701,23 @@ If you're using a L10N / I18N plugin such as L<Catalyst::Plugin::I18N> which
 provides it's own C<localize> method, you can set L<localize_from_context> to 
 use that method for formfu's localization.
 
+=head2 request_token_enable
+
+If true, adds an instance of L<HTML::FormFu::Plugin::RequestToken> to every
+form, to stop accidental double-submissions of data and to prevent CSRF attacks.
+
+=head2 request_token_field_name
+
+Defaults to C<_token>.
+
+=head2 request_token_session_key
+
+Defaults to C<__token>.
+
+=head2 request_token_expiration_time
+
+Defaults to C<3600>.
+
 =head1 DEPRECATED CONFIG SETTINGS
 
 =head2 config_file_ext
